// RealisticHealth v1.03 shader file OBGEv3 version

extern float contrast <
	string UIName = "Contrast";
	string UIWidget = "slider";
	float UIMin = 0.0f;
	float UIMax = 4.0f;
	float UIStep = 0.01f;
> = 1;

extern float brightness <
	string UIName = "Brightness";
	string UIWidget = "slider";
	float UIMin = 0.0f;
	float UIMax = 4.0f;
	float UIStep = 0.01f;
> = 1;

extern float saturation <
	string UIName = "Saturation";
	string UIWidget = "slider";
	float UIMin = 0.0f;
	float UIMax = 4.0f;
	float UIStep = 0.01f;
> = 1;

extern float radialcontrast <
	string UIName = "Radial Contrast";
	string UIWidget = "slider";
	float UIMin = -4.0f;
	float UIMax = 4.0f;
	float UIStep = 0.01f;
> = 0;

extern float radialbrightness <
	string UIName = "Radial Brightness";
	string UIWidget = "slider";
	float UIMin = -4.0f;
	float UIMax = 4.0f;
	float UIStep = 0.01f;
> = 0;

extern float radialsaturation <
	string UIName = "Radial Saturation";
	string UIWidget = "slider";
	float UIMin = -4.0f;
	float UIMax = 4.0f;
	float UIStep = 0.01f;
> = 0;

texture obge_LastRendertarget0_EFFECTPASS;

sampler currentFrameSampler = sampler_state
{
	texture = <obge_LastRendertarget0_EFFECTPASS>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MINFILTER = POINT;
	MAGFILTER = POINT;
	MIPFILTER = NONE;
};

struct VSOUT
{
	float4 vertPos : POSITION;
	float2 UVCoord : TEXCOORD0;
};

struct VSIN
{
	float4 vertPos : POSITION0;
	float2 UVCoord : TEXCOORD0;
};

VSOUT FrameVS(VSIN IN)
{
	VSOUT OUT = (VSOUT)0.0f;	// initialize to zero, avoid complaints.
	OUT.vertPos = IN.vertPos;
	OUT.UVCoord = IN.UVCoord;
	return OUT;
};

typedef half3 color3;
typedef half4 color4;

// Effect Functions
color3 doBrightness (color3 color, float value)
{
	return color * value;
}

color3 doContrast (color3 color, float value)
{
	return lerp(0.5, color, value);
}

color3 doSaturation (color3 color, float value)
{
	float luminance = dot(color3(0.299, 0.587, 0.114), color);
	return lerp(luminance, color, value);
}

color3 doGamma (color3 color, float value)
{
	return pow(color, 1.0/max(0.0001, value));
}

color3 doGammaContrast (color3 color, float value)
{
	color3 v1 = pow(color, value);
	color3 v2 = pow(1.0 - color, value);
	return v1 / (v1 + v2);
}

float getRadius(float2 UVcoord)
{
	return 1.4142 * distance(float2(0.5, 0.5), UVcoord);
}

color4 ApplyShaderEffects(float2 uv : TEXCOORD0) : COLOR0
{
	color4 color = tex2D(currentFrameSampler, uv);
	float radius = getRadius(uv);
	color.rgb = doSaturation(color.rgb, saturation + radius * radialsaturation);
	color.rgb = doContrast(color.rgb, contrast + radius * radialcontrast);
	color.rgb = doBrightness(color.rgb, brightness + radius * radialbrightness);
	return color;
}

technique T0
<
	int group = EFFECTGROUP_POST;
	int fxclass = EFFECTCLASS_COLOR;
>
{
	pass p0
	{
		VertexShader = compile vs_1_1 FrameVS();
		PixelShader = compile ps_2_0 ApplyShaderEffects();
	}
}
